 
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <limits.h>
#include <stdarg.h>
#include <string.h>
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>
#include <limits.h>
#include <dirent.h>
#include <ctype.h>
#include "error.h"
 
 
/* Kantenlaenge des Farbwuerfels */
#define MAXCOL 6
 
 
typedef struct{
        int x;
        int y;
       }store;
 
 
/* Struktur zur Spriteverwaltung */
 
#define MAXSPRITEDATA 10
 
typedef struct XR_SPRITE {
        int priority;                            /* Prioritaet des Sprites */
                                                 /* hoechste Prioritaet ist 0 */
	int width;                               /* Bildbreite */
	int height;                              /* Bildhoehe  */
	int left, top;                           /* Position des Sprites */
	int xold, yold;
	int visible;                             /* sichtbar = 1, sonst 0 */
	int showed;                              /* schon gezeigt ? */
	struct XR_SPRITE *next;                  /* Naechstes Sprite */
	struct XR_SPRITE *prev;                  /* Vorheriges Sprite */
	unsigned char *SprPtr[MAXSPRITEDATA];    /* Zeiger auf Bilddaten */
	unsigned char *MskPtr[MAXSPRITEDATA];    /* Zeiger auf Bilddaten */
        int nSprDat;                             /* Anzahl der Spritebilder */
        int actSprDat;                           /* Aktuelles Spritebild */
        int vx, vy;                              /* Geschwindigkeit in x- und y-Richtung */
        int animcount;                           /* Interner Zaehler */
        int animtime;                            /* Wartezeit zum naechsten Bild */
 
	/* X Windows spezifisch */
	XImage *im;
	Colormap colormap;
}f_x_Sprite;
 
 
/* Struktur zur Fensterverwaltung */
 
typedef struct{
       int           left, top;   /* Obere linke Fensterecke */
       int           width;       /* Fensterbreite */
       int           height;      /* Fensterhoehe */
       f_x_Sprite     *first;      /* Erstes Sprite */
 
       /* X Windows-Spezifisch */
       Window        window;
       Pixmap        pixmap;
       Pixmap        background;
       Drawable      drawable;
       Colormap      colormap;
       unsigned long colref;
       store         start;
       store         end;
      }f_x_Window;
 
 
/* Struktur zur Bildverwaltung */
 
typedef struct{
        int x, y;
	int width;                /* Bildbreite */
	int height;               /* Bildhoehe  */
	unsigned char *byteptr;   /* Zeiger auf Bilddaten */
	
	/* X Windows spezifisch */
	XImage *im;
	Colormap colormap;
}f_x_Image;
 
/* Defines fuer Mausabfrage */
#define LEFTMOUSE     1
#define RIGHTMOUSE    2
#define MIDDLEMOUSE   3
 
/* Defines fuer f_x_Rectangle und f_x_Ellipse */
#define NOTFILL 0
#define FILL   1
 
/* Defines fuer f_x_Drawmode */
#define NORMAL 0
#define XOR    1
#define AND    2
#define OR     3
 
/* Defines fuer f_x_Fade */
#define DORED   1
#define DOGREEN 2
#define DOBLUE  4
 
/* Extra Datentypen */
#define DWORD unsigned long
#define WORD  unsigned short
 
/* Fabtabelle fuer f_x_PutImage */
#define RGB 1
#define DEF 0
 
 
/* Fensterfunktionen */
 
f_x_Window *f_x_CreateWindow(int left, int top, int width, int height);
void f_x_CloseWindow( );
 
 
/* Grundfunktionen erster Teil */
 
int  f_x_Dither(int color);
void f_x_SetPixel( int x, int y);
void f_x_SetColor( int *rgb);
void f_x_RGBMode( );
void f_x_DrawMode( int mode);
int  f_x_TextHeight( );
int  f_x_TextWidth( );
void f_x_Printf( const char *format, ...);
 
 
/* Grundfunktionen zweiter Teil */
 
int error(int errnum, char *funcname, char *info);
void f_x_Move( int x, int y);
void f_x_Line( int x, int y);
void f_x_Rectangle( int left, int top,
                  int right, int bottom, int fill);
void f_x_Ellipse( int left, int top,
                  int right, int bottom, int fill);
 
 
/* Grundfunktionen dritter Teil */
 
void f_x_GetColor( int *rgb);
void f_x_GetPixel( int x, int y, int *rgb);
void f_x_Cls( );
 
 
/* Tastenfunktionen */
 
int  f_x_GetKey( );
int  f_x_WaitKey( );
 
 
/* Mausgrundfunktionen */
 
void f_x_MousePos( int *x, int *y);
int f_x_MouseButton( int which);
 
 
/* Fadeingfunktionen */
 
void f_x_FadeOut( int steps);
void f_x_Fade( unsigned char rgb_mask, float factor);
 
 
/* Bufferfunktionen */
 
void f_x_DoubleBuffer( );
void f_x_SingleBuffer( );
void f_x_SwapBuffer( );
 
 
/* Dateifunktionen */
 
int f_x_Read( char *filename, int x, int y);
 
int f_x_ReadBMP( char *filename, int x, int y);
int f_x_ReadGIF( char *filename, int x, int y);
int f_x_ReadTIFF( char *filename, int x, int y);
int f_x_ReadPCX( char *filename, int x, int y);
int f_x_WriteBMP( char *filename, int x, int y, int width, int height);
int f_x_WriteGIF( char *filename, int x, int y, int width, int height);
int f_x_WriteTIFF( char *filename, int x, int y, int width, int height);
int f_x_WritePCX( char *filename, int x, int y, int width, int height);
 
 
/* Imagefunktionen */
 
f_x_Image *f_x_CreateImage(int width, int height);
void f_x_PutImage( f_x_Image *im, int x, int y,
                 int width, int height, int steps);
f_x_Image *f_x_GetImage( int x, int y,
                 int width, int height);
void f_x_DrawImage(f_x_Image *dest, f_x_Image *src, int x, int y,
                 int l1, int l2, int sb);
void f_x_DestroyImage(f_x_Image *im);
 
 
/* Imagedateifunktionen */
 
f_x_Image *f_x_ReadImage(char *Name);
 
f_x_Image *f_x_ReadBMPImage(char *filename);
f_x_Image *f_x_ReadGIFImage(char *filename);
f_x_Image *f_x_ReadTIFFImage(char *filename);
f_x_Image *f_x_ReadPCXImage(char *filename);
int f_x_WriteBMPImage(f_x_Image *im, char *filename);
int f_x_WriteGIFImage(f_x_Image *im, char *filename);
int f_x_WriteTIFFImage(f_x_Image *im, char *filename);
int f_x_WritePCXImage(f_x_Image *im, char *filename);
 
 
/* Animationsfunktionen */
 
 
int f_x_GetFiles(char *pict_name, char *dir_name,
                 char *files[], int *nfiles);
int f_x_MakeMovie(char *filename,
                 char *files[], int nfiles);
int f_x_Animate( int show);
int f_x_OpenMovie( char *filename);
void f_x_CloseMovie(void);
void f_x_RemovePicture(void);
int f_x_InsertPicture(char *filename);
int f_x_SaveMovie(char *filename);
 
 
/* Imagegrafikfunktionen */
 
void f_x_PutPixel(f_x_Image *im, int x, int y, int rgb[]);
 
 
/* Spritefunktionen */
 
f_x_Sprite *f_x_GetSprite( char *files[], int nfiles, int pri);
void f_x_MoveSprite( f_x_Sprite *sp);
void f_x_ShowSprites( );
void f_x_ChangePriority( f_x_Sprite *sp, int pri);
void f_x_AnimateSprites( );
void f_x_StartSpriteAnimation( );
void f_x_EndSpriteAnimation( );
void f_x_DestroySprite( f_x_Sprite *sp);
 
/* Hilsfunktionen */
void get_n_bytes(unsigned char *bytepoint, unsigned char *bytes, int n);
DWORD getDWinWord(unsigned char *bytepoint, char x);
WORD getWinWord(unsigned char *bytepoint, char x);
DWORD getDWinWordFILE(FILE *pointer, char x);
WORD getWinWordFILE(FILE *pointer, char x);
void getBytes(DWORD word, unsigned char bytes[4]);
char *f_x_GetString( int x, int y, int size);
